# =============================================================================
# ------------------------- example R code - Part II --------------------------
# 
# "Sparse Bayesian modelling of underreported count data" 
# (by Dvorzak, M. and Wagner, H.)
#
# The R code is based on the R package "pogit" (available on CRAN):  
#
#           http://cran.r-project.org/web/packages/pogit/index.html 
# =============================================================================

library(pogit)
# see package?pogit for the package description


# ---------------------------------------------------------------
# Section 5: --- Application to cervical cancer data ---
# ---------------------------------------------------------------

# load cervical cancer data
data(cervical)
# see ?cervical for a description of the data set
data(cervical_validation)
# see ?cervical_validation for a description of the data set

# or alternatively, read data (electronic supplement)
# cervical <- read.table("cervical.txt", header = TRUE)
# cervical_validation <- read.table("cervical_validation.txt", header = TRUE)

y <- cervical$y
E <- cervical$E
X <- as.matrix(cervical[, -c(1:4)])
validation <- cervical_validation[, c(1, 2), drop = FALSE]
W          <- as.matrix(cervical_validation[, -c(1:3)])
subcat     <- factor(as.numeric(cervical$country))


# function call
mcmc <- list(M = 20000, burnin = 10000, startsel = 5000, thin = 10)
m3 <- pogitBvs(y = y, E = E, X = X, W = W, validation = validation, 
               model = list(subcat = subcat), mcmc = mcmc, 
               start = list(firth = TRUE), BVS = TRUE)

# print and summarize results
print(m3)
summary(m3)

# compute integrated autocorrelation times (IAT) and effective sample sizes (ESS)
# of the MCMC samples
summary(m3, IAT = TRUE)

# show traceplots of the MCMC draws
plot(m3)

# show autocorrelation plots of the MCMC draws (compare to Figure 6 - right)
plot(m3, type = "acf", lag.max = 1000)

# show model averaged posterior means and 95%-HPD intervals for regression 
# effects under the spike and slab prior (compare to Figure 4)
plot(m3, type = "hpd")

# show model averaged country- and age-specific death risk due to cervical
# cancer when accounting for underreporting (compare to Figure 5 - right)
res <- summary(m3, printRes = TRUE)$resTable
probs <- t(res$probs)
risks <- res$risks
risks100 <- risks*100 # per 100.000 woman-years

par(mfrow = c(1, 1), mar = c(4, 2, 3, 1) + 0.1)
plot(1:4, risks100[which(subcat==1)], xlim=c(0.2, 4.8), type="l", log="y", 
     xlab = NA, ylab = NA, ylim = c(0.1, 20), axes = FALSE, lwd = 2, lty = 3, 
     col = "black", pch = 4, frame.plot = F, main = "") 
points(1:4, risks100[which(subcat==2)], type = "l", lty = 1, lwd = 2, col = "black") 
points(1:4, risks100[which(subcat==3)], type = "l", lty = 6, lwd = 2, col = "black")  
points(1:4, risks100[which(subcat==4)], type = "l", lty = 2, lwd = 2, col = "black")
points(1:4, risks100[which(subcat==1)], type = "p", lty = 1, pch = 3, lwd = 1, col = "black")
points(1:4, risks100[which(subcat==2)], type = "p", lty = 1, pch = 3, lwd = 1, col = "black")
points(1:4, risks100[which(subcat==3)], type = "p", lty = 1, pch = 3, lwd = 1, col = "black")
points(1:4, risks100[which(subcat==4)], type = "p", lty = 1, pch = 3, lwd = 1, col = "black")
axis(1, at = c(0.47,1:4,4.53),tick=T, tck = -0.01, 
     labels = c("","25-34","35-44","45-54","55-64",""), cex = 0.9, line = 0)
atL     <- c(0.1, seq(0.2,1,0.1), seq(2, 10, 1), 20)
labelsL <- c("0.1", rep("", 8), "1", rep("", 8), "10", "20") 
axis(side = 2, at = atL, labels = labelsL, line = -2.8, las = 1, tck = -0.01)
segments(0.45, 10, 4.53, 10, col = "gray75")
segments(0.45, 5, 4.53, 5, col = "gray75")
segments(0.45, 1, 4.53, 1, col = "gray75")
mtext("Age (years)", side = 1, line = 2.6)
mtext(side = 2, "Deaths per 100.000 woman-years", line = -0.6)
legend("bottomright", cex = 0.6, legend = c("England", "Belgium", "France", "Italy"),
       lty = c(1,3,6,2), lwd = c(2,2,2,2), bty = "n", y.intersp = 0.5)


# function call (with MCMC settings used in the paper)
mcmc.paper <- list(M = 300000, burnin = 100000, startsel = 20000, thin = 10)
m4 <- pogitBvs(y = y, E = E, X = X, W = W, validation = validation, 
               model = list(subcat = subcat), mcmc = mcmc.paper, 
               start = list(firth = TRUE), BVS = TRUE)

#see the function calls for 'm3' to summarize and visualize the results

